# コンニャクGPT: マルチモーダルGPTを活用したより正確なアニメ字幕生成

**READMEの日本語版はChatGPTによって翻訳されました。**

[English](../README.md) [中文](./zh.md)

コンニャクGPTは、高品質かつ使いやすいAIパワードの字幕生成ツールで、主に日本のアニメ向けに開発されました。

## 主な特徴

1. **簡単に使える**: 音声から高品質の字幕ファイルを生成するための関数呼び出し一つで完了します。
2. **高品質** (予算に依存): 約20分のビデオに約4ドルかかる「中」の品質設定を使用すると、コンニャクGPTはビデオ固有の用語（キャラクター名など）の80％を正しく処理し、論理的なストーリーを持つ字幕を生成します。
   **コンニャクGPTが生成した字幕により、日本語について知識がない観客でも、プロットが理解でき、何が起こっているのかを推測する必要がありません。**
3. **翻訳サポート**: コンニャクGPTは、任意の言語の翻訳字幕を作成できます。
4. **一時停止と再開**: 字幕生成中に定期的に進行状況が保存されます。意図せず進行中の字幕生成プロセスが中断されても、進行状況を再開できます。
5. **設定可能**: コンニャクGPTは、ビデオに特殊な設定やビデオ固有の特殊用語（例：キャラクター名）が含まれる場合でも正常に動作します。ビデオの背景情報と発生する可能性のある特殊用語を入力すれば、コンニャクGPTはプロットを推論し、意味のある字幕を生成し、特殊用語を正しく処理します。

以下は、日本のアニメ「チンプイ」からのエピソードのスクリーンショットで、コンニャクGPTから生成された翻訳字幕です:

![example](../res/example.png)

コンニャクGPTは、キャラクターの名前（日本語："エリ" / 英語："Eri" / 中国語："惠理"）を正確に処理し、シーン内の特別なオブジェクトも識別し、その機能を推論しました（シーンには特定の思考を生み出す特別な光線を発射する「想像銃」があり、その効果は24時間続きます）。

## インストール

コンニャクGPTはpipを介してインストールできます:

```bash
pip install konnyaku-gpt
```

## 設定と使用方法

### Open AIの設定

**コンニャクGPTはOpen AIのAPIサービスを使用します。字幕生成を呼び出す前にAPIキーを設定する必要があります。**
最も簡単な方法は、字幕を生成するPythonスクリプトを実行する前に環境変数 "OPENAI_API_KEY" を設定することです。
例：

```bash
# Unix系のOSを使用している場合はこの行を実行します
export OPENAI_API_KEY=<あなたのAPIキー>
# Windowsを使用している場合はこの行を実行します
set OPENAI_API_KEY=<あなたのAPIキー>

python <コンニャクGPTを呼び出すスクリプト>
```

### 使用法

事前定義されたメソッドで字幕を作成するには、デフォルトの字幕生成ツールをインポートし、背景情報を入力し、ジェネレータを呼び出します。

以下は、藤子・F・不二雄が制作したアニメ「チンプイ」のエピソード用の字幕を生成するサンプルスクリプトです：

```Python
from pathlib import Path

from konnyaku_gpt.subtitle_generation import DefaultGenerator
from konnyaku_gpt.tricks import simple_split_subtitle_file

# 品質とコストのバランスを取るために、'medium'プリセットを使用することをお勧めします
generator = DefaultGenerator(quality_preset='medium')

output_path = Path('/output/srt/path')

# AIを使用して字幕を生成します！
generator.generate_subtitles(video_path=Path('/path/to/Chimpui/episode/mp4'),
                             output_path=output_path,
                             video_background=\
"""このビデオには、日本のアニメ「チンプイ」からの2つのエピソードが含まれています。
第1話のタイトルは「レッツゴー銀河レース」で、第2話のタイトルは「はじめまして、ルルロフです」です。
各エピソードの冒頭には、そのエピソードのタイトルが叫ばれる紹介画面があります。

「チンプイ」の特殊な用語:

A. キャラクター:

    1. 春日エリ（Kasuga Eri）通称「エリ」、「エリちゃん」、「エリ様」、または「春日」。彼女は少年っぽい少女で

、同級生の内木に思いを寄せており、マール星からの王子との結婚を望んでいません。
    2. チンプイ（Chimpui、中国語：芝比）：超能力を持つ宇宙のねずみ。
    3. ワンダユウ（Wanderyu、中国語：旺达）：超能力を持つ宇宙の犬。
    4. 内木翔（Shou Uchiki）通称「ウチキ」（中国語：内木）：優等生。エリが好意を寄せています。
    5. 小金山スネ美（Sunemi Koganeyama）通称「スネ美」、「スネ美ちゃん」、「スネ美さん」（中国語：诗奈美）：豪華なものや家族の富について自慢する裕福な少女。
    6. 大江山政男（Masao Oeyama）通称「オエヤマ」（中国語：大江山）：時折、同級生をいじめる強い6年生で、特にウチキをいじめます。
    7. 木常小政（Shosei Kitsune）通称「ショセイ」（中国語：小政）または「キツネ」（中国語：木常）：オエヤマの子分。
    8. 藤野ほたる（Hotaru Fujino）通称「ほたる」（中国語：小莹）：少女らしい少女で、ハンサムな王子が自分を嫁に迎えに来るなど、非現実的なことを夢見ることがよくあります。
    9. ルレアルヴ通称「ルレアルヴ殿下」（中国語：吕诺夫殿下）または単に「殿下」（殿下）：エリと結婚したいマール星の王子。しかし、エリは彼と結婚したくありません。

    チンプイとワンダは両方ともマール星出身です。
    この星は英語ではMahl planetとも呼ばれ、中国語では“玛尔星”と表記されます。

B. その他:
    1. マール（Mahl）：チンプイとワンダの出身星の名前。
    2. 科法（Kahou、科法/かほう）：マール星の特殊で先進的で便利な技術。チンプイとワンダの両方が使用します。
""",
                             target_language=None,
                             workspace_path=Path('final-episode-workspace'))

# 字幕を分割して各字幕要素が短いものにする
simple_split_subtitle_file(output_path)

```

## トラブルシューティング

### 中国ユーザー向け特別注意事項

中国国内で特定のインターネットアクセスに問題があることが知られていますので、Open AIサービスを使用するにはプロキシサーバーが必要です。
コンニャクGPT用のプロキシを設定するには、コンソールで環境変数を介してプロキシサーバーアドレスとポートを指定するだけです。例：

```bash
# Unix系のOSを使用している場合はこの行を実行します
export ALL_PROXY=127.0.0.1:<あなたのプロキシポート>
# Windowsを使用している場合はこの行を実行します
set ALL_PROXY=127.0.0.1:<あなたのプロキシポート>
```

### CUDAの設定

字幕を生成する際、コンニャクGPTはローカルに展開されたディープラーニングモデルを呼び出します。
デフォルトでは、CUDA対応のグラフィックカードがある場合、モデルをGPUに配置しようとします。
**ただし、GPUのVRAMが8GB未満の場合、モデルはVRAMに収まらない可能性があり、エラーが発生することがあります。**

この場合、CUDAを使用しないようにコンニャクGPTに指示する必要があります。最も簡単な方法は、環境変数を介してGPUをマスクすることです。つまり：

```bash
# Unix系のOSを使用している場合はこの行を実行します
export CUDA_VISIBLE_DEVICES=''
# Windowsを使用している場合はこの行を実行します
set CUDA_VISIBLE_DEVICES=''
```

## オマージュ

このプロジェクトは、有名な日本の漫画家藤本弘（Fujiko F. Fujioとしても知られる）を追悼し開発されました。
「コンニャクGPT」という名前は、彼の最も有名な作品である「ドラえもん」から着想を得ています。
